function [x,y,s,info] = sdolab(A,b,c,x0,y0,options);

%%
% [x,y,s,info] = sdolab(A,b,c,x0,y0,options);
%
% Solves a real or complex number semidefinite optimization problem in
% primal (P) or dual (D) standard forms
%
%   (P) min  c'*x               (D) max  b'*y
%       s.t. A*x = b                s.t. A'*y + s = c
%            mat(x) >= 0                 mat(s) >= 0.
%
% The data of the problem is a Hermitian matrix C of order n, an
% R-linear map Ar from Hn (the set of Hermitian matrices of order n) to
% Rmr (the set of real vectors of length mr), a real vector br in Rmr,
% an R-linear map Ac from Hn to CmcR (the set of complex vectors of
% length mc over the field of real numbers), and a complex vector bc in
% CmcR. Since the cost is computed in Sdolab by c'*x (= <C,X> where
% <A,B> is the trace of the conjugate transpose of A times B), c must
% contain C column after column. In contrast, since the constraints are
% evaluated in Sdolab by A.r*x [resp. A.c*x], the k-th constraint
% <Ark,X> [resp. <Ack,X>] must be formulated by storing the _conjugate_
% of the matrix Ark [resp. Ack] column after column in the k-th row of
% A.r [resp A.c]. Finally, br is put in b.r and bc in b.c.
%
% The unknown of the primal problem (P) is a Hermitian matrix X of order
% n stored columnwise by Sdolab in x. The constraint 'mat(x) >= 0' means
% that X = mat(x) must be positive semidefinite. The unknowns of the
% dual problem (D) are a real vector yr of length mr stored by Sdolab in
% y.r, a complex vector yc of length mc stored by Sdolab in y.c, and a
% Hermitian matrix S of order n stored columnwise by Sdolab in s. The
% constraint 'mat(s) >= 0' means that S = mat(s) must be positive
% semidefinite.
%
% An optional starting iterate (x0,y0.r,y0.c) can be proposed on entry.
% From y0, Sdolab computes an initial dual matrix S0 in the vector s0 =
% c - A.r'*y0.r - 0.5*(A.c'*y0.c+vec(mat(A.c'*y0.c)')). Next, the
% satisfaction of the affine constraints of (P) and (D) is verified. If
% this is not the case, Sdolab computes its own affine feasible starting
% point.
%
% The solver is based on the feasible Mizuno-Todd-Ye predictor-corrector
% interior-point method and the Nesterov-Todd direction.
%
% Various options allow the user to tune the solver:
% . options.cholesky: string specifying the Cholesky factorization
%   solver to use;
%   . 'chol' (default): Matlab standard solver 'chol' without pivoting
%     (the pivoting option does not seem to work), and therefore less
%     stable for large problems; works for real/complex matrices;
%   . 'chdc': Linpack solvers 'Xchdc', using pivoting and level-1 BLAS;
%     more stable for large problems; works for dense real/complex
%     matrices;
% . options.eps: required precision on the "duality gap" (default
%   1.e-8);
% . options.fout: output channel; set 1 for the standard ouput (screen,
%     default); for a specific file, use "options.fout = fopen(...)"
%     before calling Sdolab;
% . options.half_symmetric: if 'true', only the lower triangular part of
%   symmetric/hermitian matrices are built; otherwise the complete
%   matrix is built, hence repeating some computation; default is
%   'false' (has been observed to be much faster);
% . options.iter: maximal number of iterations, each iteration being a
%   combined predictor-corrector step (default 100);
% . options.max_bigm_loop: maximal big M loops, used to force
%   feasibility (default 10);
% . options.max_stepsize: largest accepted stepzise in the prediction
%   phase [probably not a good idea] (default 0.9999);
% . options.neighborhood: size of the neighborhood in which a prediction
%   phase is allowed - otherwise, only correction phases are allowed
%   (default 1/3);
% . options.nt_type: string specifying how the NT direction must be
%   computed; the possibilites are
%   . 'chol' (default): use a Cholesky factorization;
%   . 'qr': use a QR factorization;
% . options.reloop;
% . options.prediction_stepsize: specifies how the stepsize must be
%   computed in the prediction phase:
%   . 'drtv': take the deKlerk-Roos-Terlaky-Vial stepsize;
%   . 'drtv_plus': take a stepsize between the deKlerk-Roos-Terlaky-Vial
%     stepsize and options.max_stepsize clase enough to the boundary of
%     the neighborhood of the central path prescribed by
%     options.neighborhood (default);
% . options.verb: verbosity of the solver (a nonnegative number:
%   == 0: works silently;
%   >= 1: error messages (default);
%   >= 2: + initial setting, final status, and one line per iteration;
%   >= 3: more than one line per iteration + linesearch information
%   >= 4: + information that are useless for the algorithm and may be
%         more expensive to compute
%
% On return
% . x: primal solution in the form of an n^2 vector,
% . y.r: dual solution associated with the real affine constraint,
% . y.c: dual solution associated with the complex affine constraint,
% . s: dual solution associated with the positive semidefinite
%   constraint on x, in the form of an n^2 vector,
% . info gives information on the run
%   - info.flag: diagonis
%     == 0: solution found;
%     == 1: an input argument is wrong;
%     == 2: too many predictor-corrector iterations;
%     == 3: too many loops for finding the big M's;
%     == 4: primal infeasibility,
%     == 5: a mex-file is missing.
%   - info.iter: number of predictor-corrector iterations
%   - info.timer: CPU time spent in various parts of the solver
%     (1): preliminaries
%     (2): computation of the weigth matrix W
%     (3): computation of the distance to the central path
%     (4): for making the linear system
%     (5): for solving the linear system
%     (6): total CPU time
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_info SDO_options SDO_values
  global SDO_mr SDO_mc
  global SDO_mub

% Set the constant values used in Sdo

  tic_init = tic;

% Set the constant values used in Sdo

  sdolab_values ();
  SDO_info.flag = SDO_values.success;

% Set absent input arguments to empty, make global frequently used variables, and impose the entry vectors to be column vectors

  if nargin < 6, options = struct();
    if nargin < 5, y0 = [];
      if nargin < 4, x0 = [];
        if nargin < 3
          fprintf(SDO_options.fout,'\n### sdolab: the first 3 arguments are required, and c must be nonempty\n\n');
          x = x0;
          y = y0;
          info.flag = SDO_values.fail_on_argument;
          info.timer(SDO_values.cput_total) = toc(tic_init);
          return
        end
      end
    end
  end

  if isempty(options); options = struct(); end
  SDO_options = options;

  b     = b(:);
  infos = [];

% Preliminaries

  [x,y,s,eigx0,eigs0] = sdolab_prelim (A,b,c,x0,y0);

  verb = SDO_options.verb;

  if SDO_info.flag
    x = [];
    y = [];
    s = [];
    info = SDO_info;
    if verb >= 2; fprintf(SDO_options.fout,'\n%s\n',SDO_values.dline); end
    info.timer(SDO_values.cput_total) = toc(tic_init);
    return
  end

  info.timer(SDO_values.cput_prelim) = toc(tic_init);

% Choose the type of solver, depending on the primal-dual strict feasibility (sdolab_solve_infeas_bigm2 still missing)

  if eigx0 <= 0

    if verb >= 2; fprintf(SDO_options.fout,'\nPrimal variable not strictly feasible -> big M1 & M2 method'); end

    [x,y,s] = sdolab_solve_infeas_bigm1m2 (A,b,c,x,y,s,eigx0,eigs0);

  elseif eigs0 <= 0

    if verb >= 2; fprintf(SDO_options.fout,'\nDual variable not strictly feasible -> big M1 method'); end

    [x,y,s] = sdolab_solve_infeas_bigm1 (A,b,c,x,y,s,eigx0,eigs0);

  else

    [x,y,s] = sdolab_solve (A,b,c,x,y,s);

  end

% Final printings

  info = SDO_info;
  info.timer(SDO_values.cput_total) = toc(tic_init);

  if verb >= 2

    if SDO_info.flag == 0

      fprintf(SDO_options.fout,'\n%s',SDO_values.dline);
      m = SDO_mr+SDO_mc;
      cx = real(c'*x);
      by = 0;
      yv   = zeros(m,1);
      pres = zeros(m,1);
      dres = s - c;
      if SDO_mr
        yv(1:SDO_mr) = y.r;
        by = real(b.r'*y.r);
        pres(1:SDO_mr) = A.r*x-b.r;
        dres = dres + A.r'*y.r;
      end
      if SDO_mc
        yv(SDO_mr+1:m) = y.c;
        by = by + real(b.c'*y.c);
        pres(SDO_mr+1:m) = A.c*x-b.c;
        aux = A.c'*y.c;
        dres = dres + 0.5*(aux+vec(mat(aux)'));
      end
      pfeas = norm(pres);
      dfeas = norm(dres);
      fprintf(SDO_options.fout,'\n            value       |x| or |y|   equality     min eig');
      try
        fprintf(SDO_options.fout,'\nprimal  %13.6e  %11.4e  %10.4e  %10.3e',cx,norm(x),pfeas,real(eigs(mat(x),1,'sr')));
      catch
        try
          fprintf(SDO_options.fout,'\nprimal  %13.6e  %11.4e  %10.4e  %10.3e',cx,norm(x),pfeas,min(real(eig(mat(x)))));
        catch
          fprintf(SDO_options.fout,'\nprimal  %13.6e  %11.4e  %10.4e',cx,norm(x),pfeas);
        end
      end
      try
        fprintf(SDO_options.fout,'\ndual    %13.6e  %11.4e  %10.4e  %10.3e',by,norm(yv),dfeas,real(eigs(mat(s),1,'sr')));
      catch
        try
          fprintf(SDO_options.fout,'\ndual    %13.6e  %11.4e  %10.4e  %10.3e',by,norm(yv),dfeas,min(real(eig(mat(s)))));
        catch
          fprintf(SDO_options.fout,'\ndual    %13.6e  %11.4e  %10.4e',by,norm(yv),dfeas);
        end
      end
      fprintf(SDO_options.fout,'\ngap     %13.6e',cx-by);

      fprintf(SDO_options.fout,'\n%s',SDO_values.dline);
      fprintf(SDO_options.fout,'\nnb of iterations %9i',SDO_info.iter);

      fprintf(SDO_options.fout,'\n%s',SDO_values.dline);
      fprintf(SDO_options.fout,'\nTiming');
      cputot = info.timer(SDO_values.cput_total);
      if cputot
        fprintf(SDO_options.fout,'\n. preliminaries     %13.6f sec (%4.1f%%)', ...
          info.timer(SDO_values.cput_prelim),100*info.timer(SDO_values.cput_prelim)/cputot);
        fprintf(SDO_options.fout,'\n. scaling matrix W  %13.6f sec (%4.1f%%)', ...
          info.timer(SDO_values.scput_scaling_matrix_w),100*info.timer(SDO_values.scput_scaling_matrix_w)/cputot);
        fprintf(SDO_options.fout,'\n. LS make           %13.6f sec (%4.1f%%)', ...
          info.timer(SDO_values.ls_make),100*info.timer(SDO_values.ls_make)/cputot);
        fprintf(SDO_options.fout,'\n. LS solve          %13.6f sec (%4.1f%%)', ...
          info.timer(SDO_values.ls_solve),100*info.timer(SDO_values.ls_solve)/cputot);
      end
      fprintf(SDO_options.fout,'\n. total/iter        %13.6f sec',cputot/SDO_info.iter);
      fprintf(SDO_options.fout,'\n. total             %13.6f sec',cputot);
      fprintf(SDO_options.fout,'\n%s\n',SDO_values.dline);

    else

      fprintf(SDO_options.fout,'\n%s\n',SDO_values.dline);

    end

  end

% Return

end
