function [v] = sdolab_cls(M,N,p);

%%
% [v] = sdolab_cls(M,N,p);
%
% Solves the complex R-linear system in v:
%
%    M*v + N*conj(v) = p.
%
% It is assumed that M is Hermitian positive definite and that its Schur
% complement Ms := M-N*(conj(M)\conj(N)) is also Hermitian positive
% definite. The function may fail in a Cholesky factorization by lack of
% positive definiteness of the considered maatrix (this may come from
% rounding error or lack of surjectivity of the constraint map A), in
% which case it returns with SDO_info.flag = SDO_values.fail_on_chol.
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_options SDO_values

% Dimension

  mc = length(p);

% Possible options for linsolve

  optls_u         = [];
  optls_u.UT      = true;	% for U\rhs
  optls_ut        = [];
  optls_ut.UT     = true;
  optls_ut.TRANSA = true;	% for U'\rhs

% Compute the solution

  M(1:mc+1:end) = real(diag(M));				% remove imaginary scorias on the diagonal of the Hermitian matrix M

  % Cholesky factorization with 'chol' (without pivoting)

  if SDO_options.cholesky == SDO_values.cholesky_chol

    try
      Mcc = chol(conj(M));					% upper triangular Cholesky factor of conj(M)
    catch
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_cls: the Cholesky factorization of M fails (lack of positive definiteness)');
      end
      return
    end
    rhs = p - N*(linsolve(Mcc,linsolve(Mcc,conj(p),optls_ut),optls_u));	% compute the RHS of the C-linear system in v
    Ms = M - N*(linsolve(Mcc,linsolve(Mcc,conj(N),optls_ut),optls_u));	% Schur complement of M
    Ms(1:mc+1:end) = real(diag(Ms));				% remove imaginary scorias on the diagonal of Ms
    try
      Msc = chol(Ms);						% upper triangular Cholesky factor of Ms
    catch
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_cls: the Cholesky factorization of Ms fails (lack of positive definiteness)');
      end
      return
    end
    v = linsolve(Msc,linsolve(Msc,rhs,optls_ut),optls_u);	% [M - N*(conj(M)\conj(N))]*v = rhs

  else

  % Cholesky factorization with 'chdc' (with pivoting)

    [Mcc,piv,fail] = zchdc(conj(M),true);			% upper triangular Cholesky factor of the conj(M)
    if fail	% M is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_cls: the Cholesky factorization of M fails (lack of positive definiteness after %i steps on %i)',fail,mc);
      end
      return
    end
    P = sparse(double(piv),[1:mc],ones(mc,1));
    rhs = p - N*P*linsolve(Mcc,linsolve(Mcc,P'*conj(p),optls_ut),optls_u);	% compute the RHS of the C-linear system in v
    Ms = M - N*P*linsolve(Mcc,linsolve(Mcc,P'*conj(N),optls_ut),optls_u);	% Schur complement of M
    Ms(1:mc+1:end) = real(diag(Ms));				% remove imaginary scorias on the diagonal of Ms
    [Msc,piv,fail] = zchdc(Ms,true);				% upper triangular Cholesky factor of the conj(M)
    if fail	% Ms is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_cls: the Cholesky factorization of Ms fails (lack of positive definiteness after %i steps on %i)',fail,mc);
      end
      return
    end
    v(piv) = linsolve(Msc,linsolve(Msc,rhs(piv),optls_ut),optls_u);	% [M - N*(conj(M)\conj(N))]*v = rhs
    v = v(:);

  end

end
