function [d,alpha] = sdolab_nt (mr,mc,n,A,x,s,mu);

%%
% [d,alpha] = sdolab_nt (mr,mc,n,A,x,s,mu);
%
% Computes the NT direction [dx;dy;ds] at the current feasible interior
% point (x,s), using either a Cholesky factorization (if options.nt_type
% == 'chol') or a QR factorization (if options.nt_type == 'qr'). If a
% matrix that should be positive definite failed to have a Cholesky
% factorization, d is empty on return and SDO_info.flag is set to
% SDO_values.fail_on_chol.

% If nargout > 1, a stepsize alpha such that z+alpha*d in the prescribed
% neighborhood of the central path is also computed:
% - if SDO_options.prediction_stepsize == SDO_values.stepsize_drtv the
%   stepsize is the minimum of the Roos-Terlaky-Vial-deKlerk stepsize
%   and SDO_options.max_stepsize,
% - if SDO_options.prediction_stepsize == SDO_values.stepsize_drtv_plus
%   the stepsize is the largest stepsize in the interval
%   [Roos-Terlaky-Vial-deKlerk stepsize, SDO_options.max_stepsize]
%   approaching the boundery of the prescribed neighborhood of the
%   central path.
% Therefore the returned stepsize never exceeds
% SDO_options.max_stepsize.
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  d     = [];
  alpha = [];

  global SDO_info SDO_options SDO_values
  global SDO_RX SDO_RX_piv SDO_RS SDO_RS_piv SDO_D SDO_V SDO_sqrtd SDO_G SDO_W SDO_mub

% Dimensions

  n2 = n*n;
  m  = mr+mc;

% Compute GAG, the factor of the Gram matrix GAG'*GAG

  % GAG and GAHG matrices, recall that the rows of A.r and A.c contain the conjugate of the constraint matrices Ak's

  tic
  GAG  = zeros(n2,m);
  GAHG = zeros(n2,mc);
  for i = 1:mr
    GAG(:,i) = vec(SDO_G'*mat(conj(A.r(i,:)))*SDO_G);	% SDO_G'*Ai*SDO_G for i in Kr
  end
  if mc
    M = zeros(n,n);
    for i = 1:mc
      M = SDO_G'*mat(conj(A.c(i,:)))*SDO_G;
      GAG(:,mr+i) = vec(M);				% SDO_G'*Ai*SDO_G  for i in Kc
      GAHG(:,i)   = vec(M');				% SDO_G'*Ai'*SDO_G for i in Kc
    end
  end

% Compute dy

  dy.r = [];
  dy.c = [];

  % possible options for linsolve

  optls_l         = [];
  optls_l.LT      = true;	% for L\rhs
  optls_lt        = [];
  optls_lt.LT     = true;
  optls_lt.TRANSA = true;	% for L'\rhs
  optls_u         = [];
  optls_u.UT      = true;	% for U\rhs
  optls_ut        = [];
  optls_ut.UT     = true;
  optls_ut.TRANSA = true;	% for U'\rhs

  % X-mu*S^(-1)

  if mu == 0
    XmmuSinv = x;				% vec(X-mu*S^(-1))
  else
    if SDO_options.cholesky == SDO_values.cholesky_chol
      XmmuSinv = linsolve(SDO_RS,linsolve(SDO_RS,eye(n),optls_ut),optls_u);		% S^(-1)
      XmmuSinv = x-mu*vec(XmmuSinv);							% vec(X-mu*S^(-1))
    else
      PS = sparse(double(SDO_RS_piv),[1:n],ones(n,1));
      XmmuSinv = PS*linsolve(SDO_RS,linsolve(SDO_RS,full(PS'),optls_ut),optls_u);	% S^(-1)
      XmmuSinv = x-mu*vec(XmmuSinv);							% vec(X-mu*S^(-1))
    end
  end
  SDO_info.timer(SDO_values.ls_make) = SDO_info.timer(SDO_values.ls_make) + toc;

  % NT direction by Cholesky factorization

  if SDO_options.nt_type == SDO_values.nt_chol

    % Compute the matrix blocks Lij

    tic
    if SDO_options.half_symmetric
      if mr						% first row
        L11 = zeros(mr,mr);				% upper triangular matrix
        for i = 1:mr
          for j = i:mr
            L11(i,j) = real(GAG(:,i)'*GAG(:,j));	% remove small imaginary scorias
          end
        end
        if mc
          L12 = 0.5*(GAG(:,1:mr)'*GAG(:,mr+1:m));
          L13 = conj(L12);
        end
      end
      if mc						% second row
        L22 = zeros(mc,mc);
        L23 = zeros(mc,mc);
        for i = 1:mc
          for j = 1:i
            L22(i,j) = 0.5*(GAG(:,mr+i)'*GAG(:,mr+j));
            L23(i,j) = 0.5*(GAG(:,mr+i)'*GAHG(:,j));
          end
        end
      end
    else
      if mr						% first row
        L11 = real(GAG(:,1:mr)'*GAG(:,1:mr));		% remove small imaginary scorias
        if mc
          L12 = 0.5*(GAG(:,1:mr)'*GAG(:,mr+1:m));
          L13 = conj(L12);
        end
      end
      if mc						% second row
        L22 = 0.5*(GAG(:,mr+1:m)'*GAG(:,mr+1:m));
        L23 = 0.5*(GAG(:,mr+1:m)'*GAHG);
      end
    end
    SDO_info.timer(SDO_values.ls_make) = SDO_info.timer(SDO_values.ls_make) + toc;

    % Compute dy

    tic
    if SDO_options.cholesky == SDO_values.cholesky_chol	% real Cholesky factorization using 'chol' (upper triangular result)
      [L11chol,fail] = chol(L11);			% upper triangular Cholesky factor of L11
      if fail	% L11 is not positive definite
        d = [];
        SDO_info.flag = SDO_values.fail_on_chol;
        if SDO_options.verb >= 2
          fprintf(SDO_options.fout,'\nsdolab_nt: failure in the Cholesky factorization of L11 (lack of positive definiteness after %i steps on %i)',fail,mr);
        end
        return
      end
      if mc
        if mr
          M = L22 - 2*L12'*(linsolve(L11chol,linsolve(L11chol,L12,optls_ut),optls_u));
          N = L23 - 2*L12'*(linsolve(L11chol,linsolve(L11chol,L13,optls_ut),optls_u));
          p = A.c*XmmuSinv - 2*L12'*(linsolve(L11chol,linsolve(L11chol,A.r*XmmuSinv,optls_ut),optls_u));
          dy.c = sdolab_cls(M,N,p);
          if SDO_info.flag; d=[]; return; end
          dy.r = linsolve(L11chol,linsolve(L11chol,A.r*XmmuSinv-L12*dy.c-L13*conj(dy.c),optls_ut),optls_u);
        else
          p = A.c*XmmuSinv;
          dy.c = sdolab_cls(L22,L23,p);
          if SDO_info.flag; d=[]; return; end
        end
      else
        dy.r = linsolve(L11chol,linsolve(L11chol,A.r*XmmuSinv,optls_ut),optls_u);	% L11\Ar(X-mu*S^(-1))
      end
    else						% real Cholesky factorization using 'dchdc' (upper triangular result)
      [L11chol,piv,fail] = dchdc(L11,true);
      if fail	% L11 is not positive definite
        d = [];
        SDO_info.flag = SDO_values.fail_on_chol;
        if SDO_options.verb >= 2
          fprintf(SDO_options.fout,'\nsdolab_nt: failure in a Cholesky factorization of L11 (lack of positive definiteness after %i steps on %i)',fail,mr);
        end
        return
      end
      if mc
        if mr
          P = sparse(double(piv),[1:mr],ones(mr,1));
          M = L22 - 2*L12'*(P*linsolve(L11chol,linsolve(L11chol,full(P'*L12),optls_ut),optls_u));
          N = L23 - 2*L12'*(P*linsolve(L11chol,linsolve(L11chol,full(P'*L13),optls_ut),optls_u));
          p = A.c*XmmuSinv - 2*L12'*(P*linsolve(L11chol,linsolve(L11chol,full(P'*(A.r*XmmuSinv)),optls_ut),optls_u));
          dy.c = sdolab_cls(M,N,p);
          if SDO_info.flag; d=[]; return; end
          dy.r = P*linsolve(L11chol,linsolve(L11chol,full(P'*(A.r*XmmuSinv-L12*dy.c-L13*conj(dy.c))),optls_ut),optls_u);
        else
          p = A.c*XmmuSinv;
          dy.c = sdolab_cls(L22,L23,p);
          if SDO_info.flag; d=[]; return; end
        end
      else
        P = sparse(double(piv),[1:mr],ones(mr,1));
        dy.r = P*linsolve(L11chol,linsolve(L11chol,full(P'*(A.r*XmmuSinv)),optls_ut),optls_u);	% L11\Ar(X-mu*S^(-1))
      end
    end
    SDO_info.timer(SDO_values.ls_solve) = SDO_info.timer(SDO_values.ls_solve) + toc;

  % NT direction by QR factorization (calcul A VÉRIFIER - devenu faux je pense)

  else
  
    fprintf(SDO_options.fout,'\n\n### sdolab_nt: this version of the solver no longer work with the QR determination of the NT direction\n\n');
    SDO_info.flag = SDO_values.fail_on_argument;
    stop

  end

% Compute ds, dx, and d

  tic
  ds = zeros(n2,1);
  if mr; ds = -A.r'*dy.r; end
  if mc; aux = A.c'*dy.c; ds = ds - 0.5*(aux+vec(mat(aux)')); end
  dx = - XmmuSinv - vec(SDO_W*mat(ds)*SDO_W);
  d = [dx;dy.r;dy.c;ds];
  SDO_info.timer(SDO_values.ls_solve) = SDO_info.timer(SDO_values.ls_solve) + toc;

% If >1 output arguments, compute also the stepsize alpha along the predictor direction

  if nargout == 1; return; end

  % stepsize of de Klerk, Roos, Terlaky, Vial

  if SDO_options.cholesky == SDO_values.cholesky_chol
    GmdxdsG = diag(SDO_sqrtd)*(SDO_V'*linsolve(full(SDO_RX),mat(dx)*mat(ds)*SDO_G,optls_ut));
  else
    PX = sparse(double(SDO_RX_piv),[1:n],ones(n,1));
    GmdxdsG = diag(SDO_sqrtd)*(SDO_V'*linsolve(full(SDO_RX),PX'*(mat(dx)*mat(ds)*SDO_G),optls_ut));
  end
  nrm = norm(vec(GmdxdsG+GmdxdsG'));
  alpha = 2/(1+sqrt(1+6.5*nrm/SDO_mub));

  if alpha >= SDO_options.max_stepsize

    % limit alpha to SDO_options.max_stepsize

    alpha = SDO_options.max_stepsize;

  elseif SDO_options.prediction_stepsize == SDO_values.stepsize_drtv_plus

    % by the secant method, determine a stepsize close to the boundary of the prescribed neighborhood, between the drtv stepsize
    % (which is inside the 1/3 neighborhood) and SDO_options.max_stepsize (probably outside); more expensive

    alpha = SDO_options.max_stepsize;
    stop
    while (eigs(mat(x+alpha*dx),1,'sm') <= 0) | (eigs(mat(s+alpha*ds),1,'sm') <= 0)
      alpha = SDO_options.stepsize_reduction_factor*alpha;
    end
  end

end
