function [x,y,s,eigx0,eigs0] = sdolab_prelim (A,b,c,x0,y0);

%%
% [x,y,s,eigx0,eigs0] = sdolab_prelim (A,b,c,x0,y0);
%
% This function realizes the following preliminary tasks:
% - set SDO_options (default values if absent, numeric values for
%   lexical options)
% - check the given SDO_options
% - get the dimensions
% - initial printings
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_info SDO_options SDO_values
  global SDO_mr SDO_mc SDO_n SDO_n2

  x     = [];
  y     = [];
  s     = [];
  eigx0 = [];
  eigs0 = [];

% Initialize cumulative timers

  SDO_info.timer = zeros(SDO_values.cput_total,1);
  SDO_info.flag  = SDO_values.success;	% apriori, it's fine

% Some warnings off

  warning('off','MATLAB:eigs:NoEigsConverged');		% None of the 1 requested eigenvalues converged
  warning('off','MATLAB:eigs:SigmaChangedToSA');	% For real symmetric problems, sigma value 'SR' (Smallest Real) is now 'SA' (Smallest Algebraic)

% Set fout and verb

  if exist('SDO_options')
    if ~isstruct(SDO_options)
      warning('sdolab:OptionsNotStruct','argument ''options'' is expected to be a structure\n');
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if isfield(SDO_options,'fout')
      if isempty(SDO_options.fout) | (SDO_options.fout <= 0) | (fix(SDO_options.fout)-SDO_options.fout)
        if SDO_options.verb, fprintf('\n\n### sdolab: options.fout = %g not recongnized\n',SDO_options.fout); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
    end
  end
  if ~isfield(SDO_options,'fout')
    SDO_options.fout = 1;			% default output channel
  end
  fout = SDO_options.fout;

  if isfield(SDO_options,'verb')
    if isempty(SDO_options.verb) | (SDO_options.verb < 0)
      fprintf(fout,'\n\n### sdolab: options.verb must be nonnegative\n');
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.verb = 1;			% default verbosity level
  end
  verb = SDO_options.verb;			% to reduce time access

% Check options

  % Check options fields

  possible_options = {'central_point', 'cholesky', 'desired_cpdist', 'eps', 'fout', 'half_symmetric', 'iter', 'mu', 'max_bigm_loop', ...
    'max_stepsize', 'neighborhood', 'nt_type', 'prediction_stepsize', 'reloop', 'verb'};
  actual_options = fieldnames(SDO_options);
  for i=1:length(actual_options)
    if ~ismember(actual_options(i,:),possible_options)
      fprintf(fout,'\n\n### sdolab: field ''%s'' of structure ''options'' is not recognized\n',char(actual_options(i)));
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  end

  % Set verbosity level

  if isfield(SDO_options,'verb')
    if isempty(SDO_options.verb) | (SDO_options.verb < 0)
      fprintf(fout,'\n\n### sdolab: options.verb must be nonnegative\n');
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.verb = 1;			% default verbosity level: error messages
  end
  verb = SDO_options.verb;			% to reduce time access

% Decode entry arguments and check consistency

  if verb >= 2
    fprintf(fout,'\n%s',SDO_values.dline);
    fprintf(fout,'\nSDOlab solver (version 0.4, March 2017)');
    fprintf(fout,'\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^');
    fprintf(fout,'\nDimensions of the problem');
  end

% Check dimension and nonsparsity

  % analyze c and determine n

  SDO_n2 = length(c);
  SDO_n  = sqrt(SDO_n2);
  if SDO_n*SDO_n ~= SDO_n2
    if verb; fprintf(fout,'\n\n### sdolab: expecting a square number for the length of c, instead of %i\n',SDO_n2); end
    SDO_info.flag = SDO_values.fail_on_argument;
    return
  end

  if verb >= 2
    fprintf(fout,'\n. Hermitian matrix size (n) %11i',SDO_n);
  end

  if SDO_n == 0
    if verb; fprintf(fout,'\n\n### sdolab: expecting positive length for c, instead of %i\n',n); end
    SDO_info.flag = SDO_values.fail_on_argument;
    return
  end

  if issparse(c)
    if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (c)\n'); end
    SDO_info.flag = SDO_values.fail_on_argument;
    return
  end

  % analyze A

  SDO_mr = 0;
  SDO_mc = 0;

  if ~isempty(A)
    if ~isstruct(A)
      if verb; fprintf(fout,'\n\n### sdolab: expecting a structure for A\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if isfield(A,'r')
      if ~isempty(A.r)
        [SDO_mr,aux] = size(A.r);
        if aux ~= SDO_n2				% check dimension
          if verb; fprintf(fout,'\n\n### sdolab: expecting %i for the number of columns of A.r, instead of %i\n',SDO_n2,aux); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      end
      if issparse(A.r)
        if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (A.r)\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
    else
      A.r = [];		% useful locally
    end
    if isfield(A,'c')
      if ~isempty(A.c)
        [SDO_mc,aux] = size(A.c);
        if aux ~= SDO_n2				% check dimension
          if verb; fprintf(fout,'\n\n### sdolab: expecting %i for the number of columns of A.c, instead of %i\n',SDO_n2,aux); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      end
      if issparse(A.c)
        if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (A.c)\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
    else
      A.c = [];		% useful locally
    end
  end

  m = SDO_mr+SDO_mc;

  if verb >= 2
    fprintf(fout,'\n. nb of real constraints (mr) %9i',SDO_mr);
    fprintf(fout,'\n. nb of complex constraints (mc) %6i',SDO_mc);
  end

  % dimension constraint for surjectivity

  if SDO_mr+2*SDO_mc > SDO_n2
    if verb >= 2
      fprintf(fout,'\n\n### sdolab_prelim: this version of the solver does not work when mr+2*mc (%i) is larger than n^2 (%i)',SDO_mr+2*SDO_mc,SDO_n2);
      fprintf(fout,'\n### sdolab: since the constraint surjectivity cannot by satisfied\n');
    end
    SDO_info.flag = SDO_values.fail_on_argument;
    return
  end

  % analyze b

  if isempty(b)
    if m ~= 0
      if verb; fprintf(fout,'\n\n### sdolab: expecting an nonempty right hand side b\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    if m == 0
      if verb; fprintf(fout,'\n\n### sdolab: right hand side b not expected\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if ~isstruct(b)
      if verb; fprintf(fout,'\n\n### sdolab: expecting a structure for b\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if isfield(b,'r')
      if SDO_mr == 0
        if ~isempty(b.r)
          if verb; fprintf(fout,'\n\n### sdolab: right hand side b.r not expected\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      end
      if SDO_mr ~= length(b.r)
        if verb; fprintf(fout,'\n\n### sdolab: expecting b.r to be of length %i, instead of %i\n',SDO_mr,length(b.r)); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
      if issparse(b.r)
        if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (b.r)\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
    else
      if SDO_mr ~= 0
        if verb; fprintf(fout,'\n\n### sdolab: a right hand side b.r is expected\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
      b.r = [];		% useful locally
    end
    if isfield(b,'c')
      if SDO_mc == 0
        if ~isempty(b.c)
          if verb; fprintf(fout,'\n\n### sdolab: right hand side b.c not expected\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      end
      if SDO_mc ~= length(b.c)
        if verb; fprintf(fout,'\n\n### sdolab: expecting b.c to be of length %i, instead of %i\n',SDO_mc,length(b.c)); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
      if issparse(b.c)
        if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (b.c)\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
    else
      if SDO_mc ~= 0
        if verb; fprintf(fout,'\n\n### sdolab: a right hand side b.c is expected\n'); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
      b.c = [];		% useful locally
    end
  end

  % analyze x0

  x0status = 0;		% means 'x0 taken on entry'
  if ~isempty(x0)
    if length(x0) ~= SDO_n2				% check dimension
      if verb; fprintf(fout,'\n\n### sdolab: expecting %i for the length of x0, instead of %i\n',SDO_n2,length(x0)); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if issparse(x0)
      if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (x0)\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    x0n = norm(vec(x0));
    if x0n == 0; x0n = 1; end
    nonhermitian = norm(vec(mat(x0)-mat(x0)'))/x0n;	% check Hermitian symmetry
    if nonhermitian
      if nonhermitian > 1.e-10
        if verb; fprintf(fout,'\n\n### sdolab: x0 on entry is too far from being Hermitian |x0-x0''|/|x0| = %12.5e\n',nonhermitian); end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
      end
      if nonhermitian > 1.e-14
        x0 = 0.5*vec(mat(x0)+mat(x0)');
        x0status = 1;	% means 'x0 taken on entry and Hermitianized'
      end
    end
    x = x0(:);
  else
    x = [];
  end

  % analyze y0

  y.r = [];
  y.c = [];

  y0status = 0;		% means 'y0 taken on entry'
  if ~isempty(y0)
    if m == 0
      if verb; fprintf(fout,'\n\n### sdolab: dual initial variable y0 not expected\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if ~isstruct(y0)
      if verb; fprintf(fout,'\n\n### sdolab: expecting a structure for y0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
    if isfield(y0,'r')
      if SDO_mr == 0
        if ~isempty(y0.r)
          if verb; fprintf(fout,'\n\n### sdolab: dual initial variable y0.r not expected\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      else
        if SDO_mr ~= length(y0.r)
          if verb; fprintf(fout,'\n\n### sdolab: expecting y0.r to be of length %i, instead of %i\n',SDO_mr,length(y0.r)); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
        if issparse(y0.r)
          if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (y0.r)\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
        y0rn = norm(vec(y0.r));
        if y0rn == 0; y0rn = 1; end
        nonreal = norm(imag(y0.r))/y0rn;		% check real number
        if nonreal > 1.e-10
          if verb; fprintf(fout,'\n\n### sdolab: y0.r on entry is too far from being real |imag(y0.r)|/|y0.r| = %12.5e\n',nonreal); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
        y.r = real(y0.r);
      end
    end
    if isfield(y0,'c')
      if SDO_mc == 0
        if ~isempty(y0.c)
          if verb; fprintf(fout,'\n\n### sdolab: dual initial variable y0.c not expected\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
      else
        if SDO_mc ~= length(y0.c)
          if verb; fprintf(fout,'\n\n### sdolab: expecting y0.c to be of length %i, instead of %i\n',SDO_mc,length(y0.c)); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
        if issparse(y0.c)
          if verb; fprintf(fout,'\n\n### sdolab is currently not supported for sparse inputs (y0.c)\n'); end
          SDO_info.flag = SDO_values.fail_on_argument;
          return
        end
        y.c = y0.c(:);
      end
    end
  else
    y0status = 1;	% means 'y0 set to 0 by sdolab'
    y.r = zeros(SDO_mr,1);
    y.c = zeros(SDO_mc,1);
  end

% Decode options and check consistency

  if isfield(SDO_options,'central_point')
    if isempty(SDO_options.central_point)
      SDO_options.central_point = false;	% default: a solution is search, not a central point
    elseif ~islogical(SDO_options.central_point)
      if verb; fprintf(fout,'\n\n### sdolab: options.central_point must be empty or a logical variable (true/false)\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.central_point = false;		% default: a solution is search, not a central point
  end

  if isfield(SDO_options,'cholesky')
    switch lower(SDO_options.cholesky)
      case 'chol'
        SDO_options.cholesky = SDO_values.cholesky_chol;
      case 'chdc'
        SDO_options.cholesky = SDO_values.cholesky_chdc;
      otherwise
        if verb
          fprintf(fout,'\n\n### sdolab: options.cholesky = ''%s'' is not recognized',SDO_options.cholesky);
          fprintf(fout,'\n### sdolab: possible values for options.cholesky are ''chol'' (default) and ''chdc''\n');
        end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
    end
  else
    SDO_options.cholesky = SDO_values.cholesky_chol;
  end

  if isfield(SDO_options,'mu')
    if isempty(SDO_options.mu) | (SDO_options.mu <= 0)
      if verb; fprintf(fout,'\n\n### sdolab: options.mu must be > 0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.mu = 1;				% default required central parameter
  end

  if isfield(SDO_options,'desired_cpdist')
    if isempty(SDO_options.desired_cpdist) | (SDO_options.desired_cpdist <= 0)
      if verb; fprintf(fout,'\n\n### sdolab: options.desired_cpdist must be > 0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.desired_cpdist = 1.e-8;			% default required precision on mub (the gap)
  end

  if isfield(SDO_options,'eps')
    if isempty(SDO_options.eps) | (SDO_options.eps <= 0)
      if verb; fprintf(fout,'\n\n### sdolab: options.eps must be > 0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.eps = 1.e-8;			% default required precision on mub (the gap)
  end

  if isfield(SDO_options,'half_symmetric')
    if ~islogical(SDO_options.half_symmetric)
      if verb; fprintf(fout,'\n\n### sdolab: options.half_symmetric is expected to be a logical value (i.e. true or false)\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.half_symmetric = false;		% default: compute only the lower triangular part of a symmetric/hermitian matrix
  end

  if isfield(SDO_options,'iter')
    if isempty(SDO_options.iter) | (SDO_options.iter <= 0)
      if verb; fprintf(fout,'\n\n### sdolab: options.iter must be > 0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.iter = 100;			% default maximal number of iterations
  end

  if isfield(SDO_options,'max_bigm_loop')
    if isempty(SDO_options.max_bigm_loop) | (SDO_options.max_bigm_loop <= 0)
      if verb; fprintf(fout,'\n\n### sdolab: options.max_bigm_loop must be > 0\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.max_bigm_loop = 10;		% default maximal big M loops
  end

  if isfield(SDO_options,'max_stepsize')
    if isempty(SDO_options.max_stepsize) | (SDO_options.max_stepsize <= 0) | (SDO_options.max_stepsize > 1)
      if verb; fprintf(fout,'\n\n### sdolab: options.max_stepsize must be in (0,1]\n'); end
      SDO_info.flag = SDO_values.fail_on_argument;
      return
    end
  else
    SDO_options.max_stepsize = 0.9999;		% default largest stepzise
  end

  if isfield(SDO_options,'neighborhood')
    if SDO_options.neighborhood <= 0 || SDO_options.neighborhood > 1/3
      if verb; fprintf(fout,'\n\n### sdolab: options.neighborhood = ''%9.2e'' is not in (0,1/3], reset to 1/3\n',SDO_options.neighborhood); end
      SDO_options.neighborhood = 1/3;
    end
  else
    SDO_options.neighborhood = 1/3;
  end

  if isfield(SDO_options,'nt_type')
    switch lower(SDO_options.nt_type)
      case 'chol'
        SDO_options.nt_type = SDO_values.nt_chol;
      case 'qr'
        SDO_options.nt_type = SDO_values.nt_qr;
      otherwise
        if verb
          fprintf(fout,'\n\n### sdolab: options.nt_type = ''%s'' is not recognized',SDO_options.nt_type);
          fprintf(fout,'\n### sdolab: possible values for options.nt_type are ''fixed'', ''constraint'', and ''constraint-change''\n');
        end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
    end
  else
    SDO_options.nt_type = SDO_values.nt_chol;
  end

  if isfield(SDO_options,'prediction_stepsize')					% prediction stepsize type
    switch lower(SDO_options.prediction_stepsize)
      case 'drtv'
        SDO_options.prediction_stepsize = SDO_values.stepsize_drtv;		% deKlerk-Roos-Terlaky-Vial stepsize
      case 'drtv_plus'
        SDO_options.prediction_stepsize = SDO_values.stepsize_drtv_plus;	% stepsize larger than the deKlerk-Roos-Terlaky-Vial stepsize
      otherwise
        if verb
          fprintf(fout,'\n\n### sdolab: options.prediction_stepsize = ''%s'' is not recognized',SDO_options.prediction_stepsize);
          fprintf(fout,'\n### sdolab: possible values for options.prediction_stepsize are ''drtv'' and ''drtv_plus''\n');
        end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
    end
  else
    SDO_options.prediction_stepsize = SDO_values.stepsize_drtv;		% default is 'drtv'
  end

  if isfield(SDO_options,'reloop')
    switch lower(SDO_options.reloop)
      case 'warm'
        SDO_options.reloop = SDO_values.reloop_warm;
      case 'cold'
        SDO_options.reloop = SDO_values.reloop_cold;
      otherwise
        if verb
          fprintf(fout,'\n\n### sdolab: options.reloop = ''%s'' is not recognized',SDO_options.reloop);
          fprintf(fout,'\n### sdolab: possible values for options.reloop are ''cold'' and ''warm''\n');
        end
        SDO_info.flag = SDO_values.fail_on_argument;
        return
    end
  else
    SDO_options.reloop = SDO_values.reloop_cold;
  end

  if verb >= 2
    fprintf(fout,'\nTuning');
    fprintf(fout,'\n. verbosity level %21i',verb);
    if SDO_options.half_symmetric
      fprintf(fout,'\n. lower triangular computation of symmetric matrices');
    else
      fprintf(fout,'\n. full computation of symmetric matrices');
    end
    if SDO_options.cholesky == SDO_values.cholesky_chdc
      fprintf(fout,'\n. Cholesky factorizations using ''chdc''');
    else
      fprintf(fout,'\n. Cholesky factorizations using ''chol''');
    end
    if SDO_options.central_point
      fprintf(fout,'\n. only a central point is searched');
      fprintf(fout,'\n. desired central parameter           %9.3e',SDO_options.mu);
      fprintf(fout,'\n. desired central proximity           %9.3e',SDO_options.desired_cpdist);
    else
      fprintf(fout,'\n. required precision on duality gap   %9.3e',SDO_options.eps);
      fprintf(fout,'\n. neighborhood size for prediction    %9.3e',SDO_options.neighborhood);
      fprintf(fout,'\n. max steplength in prediction        %9.3e',SDO_options.max_stepsize);
      if SDO_options.prediction_stepsize == SDO_values.stepsize_drtv
        fprintf(fout,'\n. prediction stepsize of de Klerk, Roos, Terlaky, and Vial');
      else
        fprintf(fout,'\n. prediction stepsize close to the neighborhodd boundary');
      end
    end
  end

% Check the strict fesibility of the intial (x,y)

  if verb >= 2
    fprintf(fout,'\nInitial iterate');
  end

  if m

    bn = max(norm(b.r,Inf),norm(b.c,Inf));
    if bn == 0; bn = 1; end

    if isempty(x)
      x0status = 2;	% means 'x0 computed by sdolab'
    else
      prn = 0;
      if SDO_mr; prn = norm(A.r*x-b.r,Inf); end
      if SDO_mc; prn = max(prn,norm(A.c*x-b.c,Inf)); end
      rprn0 = prn/bn;
      if rprn0 > 1.e-10
        x = [];
        x0status = 3;	% means 'x0 recomputed by sdolab'
      end
      rprn = rprn0;
    end

    if isempty(x)
      x = sdolab_x0(A,b,x0);
      if isempty(x)
        if verb; fprintf(fout,'\n\nsdolab: the problem is primal infeasible (incompatible affine constraints A*x=b)\n'); end
        SDO_info.flag = SDO_values.stop_on_primal_infeasibility;
        return
      end
      prn = 0;
      if SDO_mr; prn = norm(A.r*x-b.r,Inf); end
      if SDO_mc; prn = max(prn,norm(A.c*x-b.c,Inf)); end
      rprn = prn/bn;
    end

    if verb >= 2
      fprintf(fout,'\n. |A*x0-b|/|b| = %12.5e',rprn);
    end

  end

  if verb >= 2
    switch x0status
      case 0
        fprintf(fout,' (x0 taken on entry)');
      case 1
        fprintf(fout,' (x0 taken on entry and Hermitianized)');
      case 2
        fprintf(fout,' (x0 computed by sdolab)');
      case 3
        fprintf(fout,' (x0 = projection of x0_entry, since too large relative residual [=%10.4e])',rprn0);
    end
  end

  if SDO_mc == 0
    eigx0 = min(real(eig(mat(x))));
  else
    try
      eigx0 = eigs(mat(x),1,'sr');	% one eigenvalue with the smallest real part (all the eignevalues are real, actually)
    catch
      eigx0 = min(real(eig(mat(x))));
    end
  end
  if verb >= 2; fprintf(fout,'\n. min(eig(x0)) = %12.5e',eigx0); end

  if SDO_mr
    s = c - A.r'*y.r;
  end
  if SDO_mc
    aux = A.c'*y.c;
    s = s - 0.5*(aux+vec(mat(aux)'));
  end

  if SDO_mc == 0
    eigs0 = min(real(eig(mat(s))));
  else
    try
      eigs0 = eigs(mat(s),1,'sr');	% one eigenvalue with the smallest real part (all the eignevalues are real, actually)
    catch
      eigs0 = min(real(eig(mat(s))));
    end
  end
  if verb >= 2
    fprintf(fout,'\n. min(eig(s0)) = %12.5e',eigs0);
    switch y0status
      case 0
        fprintf(fout,' (y0 taken on entry)');
      case 1
        fprintf(fout,' (y0 set to 0 by sdolab)');
    end
  end

% Verify the presence of mexfiles

  if SDO_options.cholesky == SDO_values.cholesky_chdc
    if exist(sprintf('dchdc.%s',mexext)) ~= 3
      fprintf(fout,'\n\n### sdolab: go in the ''sdolab'' directory and type ''makemex'' to generate ''dchdc.%s''\n',mexext)
      SDO_info.flag = SDO_values.stop_on_mexfile;
      info = SDO_info;
      return
    end
    if exist(sprintf('zchdc.%s',mexext)) ~= 3
      fprintf(fout,'\n\n### sdolab: go in the ''sdolab'' directory and type ''makemex'' to generate ''zchdc.%s''\n',mexext)
      SDO_info.flag = SDO_values.stop_on_mexfile;
      info = SDO_info;
      return
    end
  end

% Exit

  if verb >= 2
    fprintf(fout,'\n%s',SDO_values.dline);
  end

  return
