function [x,y,s] = sdolab_solve (A,b,c,x,y,s);

%%
% [x,y,s] = sdolab_solve (A,b,c,x,y,s);
%
% Solve an SDOlab problem from a strictly feasible point.
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_info SDO_options SDO_values
  global SDO_mub

% Dimensions

  n2 = length(c);
  n  = sqrt(n2);
  if isfield(A,'r')
    mr = size(A.r,1);
  else
    b.r = [];	% useful locally
    mr = 0;
  end
  if isfield(A,'c')
    mc = size(A.c,1);
  else
    b.c = [];	% useful locally
    mc = 0;
  end
  m = mr+mc;

% Some constants

  verb = SDO_options.verb;

  onethird = 1/3;	% size of the neighborhood where predictor-corrector steps can be done (otherwise do corrector steps)
  tau = 0.99;		% rate of linear convergence required on the distance to the CP in the initial phase
  nolinesearch = 2*tau*tau;
  nolinesearch = sqrt(nolinesearch/(1+nolinesearch));
  omega = 1.e-4;	% for the linesearch

% Check affine feasibility on entry

  if m

    if verb >= 2; fprintf(SDO_options.fout,'\nAffine feasibility: '); end

    prn = 0;
    if mr; prn = norm(A.r*x-b.r,Inf); end
    if mc; prn = max(prn,norm(A.c*x-b.c,Inf)); end
    bn = max(norm(b.r,Inf),norm(b.c,Inf));

    if bn == 0
      if verb >= 2; fprintf(SDO_options.fout,'primal %8.2e (abs), ',prn); end
    else
      prn = prn/bn;
      if verb >= 2; fprintf(SDO_options.fout,'primal %8.2e (rel, |b| = %8.2e), ',prn,bn); end
    end
    if prn > 1.e-10
      fprintf(SDO_options.fout,'\n\n### sdolab: affine primal feasibility fails\n\n');
      SDO_info.flag = SDO_values.linear_primal_infeasibility;
      return
    end

  end

  dres = s - c;
  if mr; dres = dres + A.r'*y.r; end
  if mc; aux = A.c'*y.c; dres = dres + 0.5*(aux+vec(mat(aux)')); end
  cn = norm(c);
  if cn == 0
    dn = norm(dres);
    if verb >= 2; fprintf(SDO_options.fout,'dual %8.2e (abs)',dn); end
  else
    dn = norm(dres)/cn;
    if verb >= 2; fprintf(SDO_options.fout,'dual %8.2e (rel)',dn); end
  end
  if dn > 1.e-10
    fprintf(SDO_options.fout,'\n\n### sdolab: dual affine feasibility fails\n\n');
    SDO_info.flag = SDO_values.linear_dual_infeasibility;
    return
  end

% Main loop

  if verb == 2; fprintf(SDO_options.fout,'\nit       c''*x          gap     cp dist   stepsize  type'); end

  z = [x;y.r;y.c;s];

  SDO_info.iter = 0;

  while 1	% optimization loop

    % increase the iteration counter

    SDO_info.iter = SDO_info.iter+1;
    if SDO_info.iter > SDO_options.iter
      if verb >= 2; fprintf(SDO_options.fout,'\n\n### sdolab: max number of iterations reached\n'); end
      SDO_info.flag = SDO_values.stop_on_iter;
      return
    end

    % compute G and W (in globals)

    tic;
    sdolab_gw (n,x,s);
    if SDO_info.flag == SDO_values.fail_on_chol; break; end
    SDO_info.timer(SDO_values.scput_scaling_matrix_w) = SDO_info.timer(SDO_values.scput_scaling_matrix_w) + toc;

    % distance to the central path

    tic;
    cpdist = sdolab_cpdist(x,s);
    SDO_info.timer(SDO_values.cput_cpdist) = SDO_info.timer(SDO_values.cput_cpdist) + toc;

    % printings

    if verb >= 2
      if verb == 2
        SDO_info.obj = c'*x;
        fprintf(SDO_options.fout,'\n%2i  %14.7e',SDO_info.iter,SDO_info.obj);
        fprintf(SDO_options.fout,'  %9.3e  %7.1e',SDO_mub,cpdist);
      else
        SDO_info.obj = c'*x;
        fprintf(SDO_options.fout,'\n\niter %i,  c''*x %14.7e,  gap %9.3e,  cp-dist %7.1e',SDO_info.iter,SDO_info.obj,SDO_mub,cpdist);
      end
    end

    % predictor phase if z in cpdist <= 1/3 (and ~SDO_options.central_point)

    if ~SDO_options.central_point & (cpdist <= onethird)

      mu = 0;
      [d,alpha] = sdolab_nt(mr,mc,n,A,x,s,mu);
      if isempty(d); break; end
      alpha = min(alpha,SDO_options.max_stepsize);
      z = z+alpha*d;

      if verb == 2
        fprintf(SDO_options.fout,'  %10.4e  pc',alpha);
      elseif verb > 2
        if verb == 3
          fprintf(SDO_options.fout,'\n. predictor phase with stepsize %10.4e',alpha);
        else
          fprintf(SDO_options.fout,'\n. predictor phase\n  > stepsize                     %10.4e',alpha);
          x = z(1:n2);
          pres = zeros(m,1);
          if mr; pres(1:mr)   = A.r*x-b.r; end
          if mc; pres(mr+1:m) = A.c*x-b.c; end
          if bn == 0
            prn = norm(pres);
            fprintf(SDO_options.fout,'\n  > resulting primal feasibility %10.4e (abs), ',prn);
          else
            prn = norm(pres)/bn;
            fprintf(SDO_options.fout,'\n  > resulting primal feasibility %10.4e (rel), ',prn);
          end
        end
      end

    end

    % corrector phase (always)

    x = z(1:n2);
    s = z(n2+m+1:n2+m+n2);
    sdolab_gw (n,x,s);			% G, W, and mub depend on X and S and must be updated
    if SDO_info.flag == SDO_values.fail_on_chol; break; end
    if SDO_options.central_point
      mu = SDO_options.mu;
    else
      mu = SDO_mub;
    end
    d = sdolab_nt (mr,mc,n,A,x,s,mu);
    if isempty(d); break; end

    if cpdist <= onethird

      z = z+d;
      if (verb == 2) & SDO_options.central_point
        fprintf(SDO_options.fout,'  %10.4e   c',1.e0);
      end
      if verb > 2
        fprintf(SDO_options.fout,'\n. corrector phase with unit stepsize (since cp-dist < 1/3)');
      end

    else

      if cpdist <= nolinesearch
        z = z+d;
        if verb == 2
          fprintf(SDO_options.fout,'  %10.4e   c',1.e0);
        elseif verb > 2
          fprintf(SDO_options.fout,'\n. corrector phase with unit stepsize (since cp-dist < %8.2e)',nolinesearch);
        end
      else
        z0    = z;
        phi0  = x'*s - mu*real(log(det(mat(x)))+log(det(mat(s))));	% discard small imaginary scorias
        itls  = 0;
        alpha = 1;
        slope = -4*mu*cpdist*cpdist;
        if verb > 2
          fprintf(SDO_options.fout,'\n. corrector phase with linesearch');
          fprintf(SDO_options.fout,'\n  iter  stepsize      merit      slope->estim');
          fprintf(SDO_options.fout,'\n  %4i            %13.6e  %12.5e',0,phi0,slope);
        end
        coef  = omega*slope;
        while 1
          itls = itls+1;
          z = z0 + alpha*d;
          x = z(1:n2);
          try	% eigs often fails to find the smallest eignvalue in difficult cases
            if eigs(mat(x),1,'sr') <= 0
              if verb > 2; fprintf(SDO_options.fout,'\n  %4i  %8.2e  %13.6e  %12.5e',itls,alpha,Inf,Inf); end
              alpha = 0.5*alpha;
              continue
            end
          catch % let us try by finding all the eigenvalues
            if min(real(eig(mat(x)))) <= 0
              if verb > 2; fprintf(SDO_options.fout,'\n  %4i  %8.2e  %13.6e  %12.5e',itls,alpha,Inf,Inf); end
              alpha = 0.5*alpha;
              continue
            end
          end
          s = z(n2+m+1:n2+m+n2);
          try	% eigs often fails to find the smallest eignvalue in difficult cases
            if eigs(mat(s),1,'sr') <= 0
              if verb > 2; fprintf(SDO_options.fout,'\n  %4i  %8.2e  %13.6e  %12.5e',itls,alpha,Inf,Inf); end
              alpha = 0.5*alpha;
              continue
            end
          catch	% let us try by finding all the eigenvalues
            if  min(real(eig(mat(s)))) <= 0
              if verb > 2; fprintf(SDO_options.fout,'\n  %4i  %8.2e  %13.6e  %12.5e',itls,alpha,Inf,Inf); end
              alpha = 0.5*alpha;
              continue
            end
          end
          phi = x'*s - mu*real(log(det(mat(x)))+log(det(mat(s))));	% discard small imaginary scorias
          if verb > 2; fprintf(SDO_options.fout,'\n  %4i  %8.2e  %13.6e  %12.5e',itls,alpha,phi,(phi-phi0)/alpha); end
          if phi <= phi0 + coef*alpha; break; end
          alpha = 0.5*alpha;
        end
        if verb == 2; fprintf(SDO_options.fout,'  %10.4e   c',alpha); end
      end
    end

    % new (x,s)

    x = z(1:n2);
    s = z(n2+m+1:n2+m+n2);

    % stopping test

    if SDO_options.central_point
      if cpdist <= SDO_options.desired_cpdist
        y.r = z(n2+1:n2+mr);
        y.c = z(n2+mr+1:n2+m);
        SDO_info.flag = SDO_values.success;
        break
      end
    else
      if SDO_mub <= SDO_options.eps
        y.r = z(n2+1:n2+mr);
        y.c = z(n2+mr+1:n2+m);
        SDO_info.flag = SDO_values.success;
        break
      end
    end

  end

% Here, it is supposed that the (x,y,s) to be returned is known

  SDO_info.obj = c'*x;
  newmu = x'*s/n;

  if verb >= 2
    if SDO_info.flag == SDO_values.success
      if SDO_options.central_point
        fprintf(SDO_options.fout,'\nCentral point found up to the required precision (%g)',SDO_options.desired_cpdist);
      else
        if verb == 2
          fprintf(SDO_options.fout,'\n%2i  %14.7e  %9.3e',SDO_info.iter+1,SDO_info.obj,SDO_mub);
        elseif verb > 2
          fprintf(SDO_options.fout,'\n\niter %i,  c''*x %14.7e,  gap %9.3e\n',SDO_info.iter+1,SDO_info.obj,SDO_mub);
        end
        fprintf(SDO_options.fout,'\nSolution found up to the required precision (%g)',SDO_options.eps);
      end
    end
  end

% Return

  return

end
