function [x,y,s] = sdolab_solve_infeas_bigm1m2 (A,b,c,x,y,s,eigx0,eigs0);

%%
% [x,y,s] = sdolab_solve_infeas_bigm1m2 (A,b,c,x,y,s,eigx0,eigs0);
%
% Solve an SDO problem from an arbitrary (not necessarily feasible)
% point, using the big M method for forcing strict primal-dual
% feasibility.
%
% It is assumed that (y,s) is dual feasible and that eigx0 = min(eig(x))
% and that eigs0 = min(eig(s)).
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_info SDO_options SDO_values

% Dimensions and constants

  if isfield(A,'r') & ~isempty(A.r)
    [mr,n2] = size(A.r);
  else
    mr = 0;
  end
  if isfield(A,'c') & ~isempty(A.c)
    [mc,n2] = size(A.c);
  else
    mc = 0;
  end
  n = sqrt(n2);

  verb = SDO_options.verb;

% Modify the problem

  % modify A (independent of M1 and M2)

  mrt = mr+1;
  mct = mc+2*n+1;
  nt  = n+2;
  nt2 = nt*nt;

  if verb >= 2
    fprintf(SDO_options.fout,'\nDimensions of the modified problem');
    fprintf(SDO_options.fout,'\n. Hermitian matrix size (n) %11i',nt);
    fprintf(SDO_options.fout,'\n. nb of real constraints (mr) %9i',mrt);
    fprintf(SDO_options.fout,'\n. nb of complex constraints (mc) %6i',mct);
  end

  if SDO_options.reloop == SDO_values.reloop_warm
    fprintf(SDO_options.fout,'\n. with warm');
  else
    fprintf(SDO_options.fout,'\n. with cold');
  end
  fprintf(SDO_options.fout,' loop restarts');

  At.r = zeros(mrt,nt2);
  ArI = A.r*vec(eye(n));
  for i = 1:mr
    At.r(i,:) = vec([mat(A.r(i,:))  zeros(n,2)
                     zeros(1,nt)
                     zeros(1,n+1)   -ArI(i)]);
  end
  At.r(mr+1,:) = vec([-speye(n)    zeros(n,2)
                      zeros(1,n)   -1        0
                      zeros(1,nt)]);

  At.c = zeros(mct,nt2);
  if mc
    AcI = A.c*vec(eye(n));
    for i = 1:mc
      At.c(i,:) = vec([mat(A.c(i,:))  zeros(n,2)
                       zeros(1,nt)
                       zeros(1,n+1)   -AcI(i)]);
    end
  end
  zzz = zeros(nt,nt);
  nc = mc;
  for j = 1:n
    nc = nc+1;
    zzz(j,n+1) = 1;
    At.c(nc,:) = vec(zzz);
    zzz(j,n+1) = 0;
  end
  for j = 1:n+1
    nc = nc+1;
    zzz(j,n+2) = 1;
    At.c(nc,:) = vec(zzz);
    zzz(j,n+2) = 0;
  end

  % Initialize the loop on M1 and M2

  ic  = trace(mat(c));	% <I,C>
  ix  = trace(mat(x));	% <I,X>
  is  = trace(mat(s));	% <I,S>

  % initial xt
  x2  = max(-eigx0,0);
  x2  = max(x2+1,2*x2);
  tmp = ix+n*x2;
  M1  = max(tmp+1,2*tmp);
  x1  = M1-tmp;
  xt  = vec([mat(x)+x2*speye(n)  zeros(n,2)
             zeros(1,n)          x1          0
             zeros(1,n+1)                    x2]);

  % initial yt
  t    = max(-eigs0,0);	% it is assumed that (y,s) on entry is dual feasible
  t    = max(t+1,2*t);
  yt.r = [y.r; t];
  yt.c = [y.c; zeros(2*n+1,1)];

  % initial st
  tmp = max(is,0);
  M2  = max(tmp+1,2*tmp);
  s2  = M2-is;
  st  = vec([mat(s)+t*speye(n)  zeros(n,2)
              zeros(1,n)        t           0
              zeros(1,n+1)                  s2]);

  % loop on M1 and M2

  nloop = 0;

  while 1

    nloop = nloop+1;

    if verb >= 2

      fprintf(SDO_options.fout,'\n%s',SDO_values.dline);
      fprintf(SDO_options.fout,'\nLoop %i,  M1 = %11.5e,  M2 = %11.5e',nloop,M1,M2);

      if nloop > 1
        if SDO_options.reloop == SDO_values.reloop_warm
          fprintf(SDO_options.fout,'  (warm start)');
        else
          fprintf(SDO_options.fout,'  (cold start)');
        end
      end

    end

    ct = vec([mat(c)       zeros(n,2)
             zeros(1,n+2)
             zeros(1,n+1)  M2-ic]);
    bt.r = [b.r; -M1];
    if mc
      bt.c = [b.c; zeros(2*n+1,1)];
    else
      bt.c = [zeros(2*n+1,1)];
    end

    [xtnew,ytnew,stnew] = sdolab_solve (At,bt,ct,xt,yt,st);

    switch SDO_info.flag
    case {SDO_values.linear_primal_infeasibility, SDO_values.linear_dual_infeasibility, SDO_values.stop_on_iter}
      break
    case SDO_values.fail_on_chol
      if SDO_info.iter <= 1; break; end
    end

    Xt = mat(xtnew);
    St = mat(stnew);
    pshift = Xt(end,end);
    dshift = St(n+1,n+1);

    if verb >= 2
      fprintf(SDO_options.fout,'\nPrimal shift = %11.5e,  dual shift = %11.5e',pshift,dshift);
    end

    if (SDO_info.flag == SDO_values.success) & (pshift < 1.e-10) && (dshift < 1.e-10)
      x = vec(Xt(1:n,1:n));
      y.r = ytnew.r(1:mr);
      y.c = ytnew.c(1:mc);
      s = vec(St(1:n,1:n));
      break
    end

    if SDO_options.reloop == SDO_values.reloop_warm

      if pshift > 1.e-10	% increase M2 and s2
        M2old = M2;
        M2 = M2*10;
        St(n+2,n+2) = St(n+2,n+2) + (M2-M2old);
        st = vec(St);
      end
      if dshift > 1.e-10	% increase M1 and x1
        M1old = M1;
        M1 = M1*10;
        Xt(n+1,n+1) = Xt(n+1,n+1) + (M1-M1old);
        xt = vec(Xt);
      end
      yt = ytnew;

    else

      if pshift > 1.e-10	% increase M2 and adapt st
        M2 = M2*10;
        s2 = M2-is;
        st = vec([mat(s)+t*eye(n)  zeros(n,2)
                  zeros(1,n)       t           0
                  zeros(1,n+1)                 s2]);
      end
      if dshift > 1.e-10	% increase M1 and adapt xt
        M1 = M1*10;
        x1 = M1-(ix+n*x2);
        xt = vec([mat(x)+x2*eye(n)  zeros(n,2)
                  zeros(1,n)        x1          0
                  zeros(1,n+1)                  x2]);
      end

    end

    if nloop >= SDO_options.max_bigm_loop
      fprintf(SDO_options.fout,'\n\n### sdolab: too many big M loops''s\n');
      SDO_info.flag = SDO_values.stop_on_bigm_loops;
      break
    end

  end

% Return

  return

end
