function [x0] = sdolab_x0(A,b,x0);

%%
% [x0] = sdolab_x0(A,b);
%
% Computes a Hermitian x0 such that A*x0 = b. If this is not possible
% (since b in not in the range space of A), x0 is empty on return.
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_mr SDO_mc SDO_n SDO_n2

% Dimensions

  mm = SDO_mr+2*SDO_mc;

% The easy case when b == 0

  bb  = [b.r;real(b.c);imag(b.c)];	% extended real b
  bbn = norm(bb);
  if bbn == 0
    x0 = zeros(SDO_n2,1);
    return
  end

% AH = row by row transpose-conjugate of A.c

  if SDO_mc
    AH = zeros(SDO_mc,SDO_n2);
    for  k=1:SDO_mc
      AH(k,:) = vec(mat(A.c(k,:))');
    end
  else
    AH = [];
  end

% Compute the QR factorization of an extended A (expensive ...). Mind the -0.5i*(A.c-AH) instead of 0.5i*(A.c-AH) since the k-th
% matrix row must contained the conjugate of 0.5*i(Ak-Ak') and that the rows of A.c and AH already contain the conjugate of Ak
% and Ak'.

  AA    = [A.r;0.5*(A.c+AH);-0.5i*(A.c-AH)];
  [Q,R] = qr(AA);	% inherit sparsity

  % compute the rank of A

  r = mm;
  while r>0 && norm(R(r,:))==0
    r = r-1;
  end

% Infeasibility if b is is not in the range space of A

  if r < mm
    Z = Q(:,r+1:mm);			% R(Z) = Null(A')
    if norm(Z*(Z'*bb))/bbn > 1.e+3*eps	% the orthogonal projection Z*Z'*b of b on the space orthogonal to Range(A) is nonzero
      x0 = [];				% then A*x = b is infeasible
      return
    end
  end

% Compute x0

% x0 = R\(Q(:,1:r)'*bb);		% fail to give the rigth solution in the Matpower problem caseWB5

  if isempty(x0)
    x0 = AA\bb;				% very crude, but probably more efficient in this compact Matlab expression
  else
    dx = AA\(bb-AA*x0);			% very crude, but probably more efficient in this compact Matlab expression
    x0 = x0+dx;
  end
  x0 = vec(0.5*(mat(x0)+mat(x0)'));	% make the matrix Hermitian

% Return

  return

end
