#include "fintrf.h"

c=======================================================================
c     
c  This Fortran file contains two parts: the gateway subroutine
c  'mexFunction' and the Linpack subroutine 'zchdc' (together with its
c  level-1 BLAS dependencies zaxpy, zswap, and dcabs1).
c
c  - The gateway subroutine 'mexFunction' makes the link between the
c    Matlab memory space and the Fortran memory space. It transforms the
c    data from the Matlab space into the one expected by 'zchdc' and
c    after execution of this one, it reformats the output of the
c    subroutine and store it in the Matlab space.
c
c  - The subroutne 'zchdc' is a LINPACK subroutine that computes the
c    Cholesky factorization with possible pivoting of a complex positive
c    definite matrix.
c
c  To be able to use zchdc in Matlab, first generate the object code
c  (Mex file) for your machine by typing 'mex zchdc.F' in the Matlab
c  window.
c
c  In Matlab, zchdc is called by
c
c     [R,p,info] = zchdc(A,withpivoting)
c
c  where
c  - A is the order n positive definite Hermitian complex matrix to
c    factor,
c  - withpivoting must be true (not the double precision constant 1) if
c    pivoting is required or must be false (not the double precision
c    constant 0) otherwise,
c  - R is an upper triangular matrix such that (in Matlab)
c
c      R'*R = A(p,p),
c
c  - p is the n integer vector of pivots,
c  - info specifies whether the factorization has been processed
c    correctly (0 if this is the case and a positive number in [1:n-1]
c    specifying the last stage of the factorization otherwise).
c
c  If R is given by the factorization above with pivoting, the solution
c  x to a single linear system A*x=b can be obtained in Matlab by
c
c     x(p) = R\(R'\b(p));
c
c  (use linsolve instead of "\" for more efficiency, since R is upper
c  triangular). The solution to a set of linear systems A*X=B can be
c  obtained by
c
c     n = size(R,1);
c     P = sparse(double(p),[1:n],ones(n,1));
c     X = P*(R\(R'\(P'*B)));
c     
c
c  The procedure does not work on sparse or real data. For real data,
c  use 'dchdc'.
c
c  Author of the gateway: J.Ch. Gilbert, Inria, March 2017.
c
c=======================================================================

      subroutine mexFunction (nlhs, plhs, nrhs, prhs)

c --- Argument Declaration: plhs (resp. prhs) is a vector of pointers to
c     the output or left-hand-side variables (resp. input or
c     right-hand-side variables) of length nlhs (resp. rlhs). In other
c     words, the function zchdc in Matlab has nlhs output variables
c     and nrhs input variables.

      implicit none
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

c --- Function declarations

      mwPointer mxGetPr, mxGetPi
      mwPointer mxCreateDoubleMatrix, mxCreateNumericMatrix
      integer*4 mxClassIDFromClassName
      mwPointer mxGetM

c --- Pointers to input/output mxArrays in the Matlab space

      mwPointer p_ptr, info_ptr, pivot_ptr

c --- Other variables. Luckily, allocation of variables in the Fortran
c     space is possible, using 'allocate' like below, and this must be
c     declared like in Fortran 20xx (note that integer in Fortran =
c     integer*4 in Fortran = int32 in Matlab)

      integer i, j, n, n2, job, info
      integer*4 classid
      integer, dimension(:), allocatable :: jpvt
      complex*16, dimension(:), allocatable :: work
      complex*16, dimension(:,:), allocatable :: a

c-----------------------------------------------------------------------

c --- Check correctedness of the arguments. 

      if (nrhs .ne. 2) then
         call mexErrMsgIdAndTxt ('MATLAB:zchdc:nInput',
     +                           'Two input arguments required.')
      elseif (nlhs .ne. 3) then
         call mexErrMsgIdAndTxt ('MATLAB:zchdc:nOutput',
     +                           'Three output arguments required.')
      endif

c --- Get the order n of A and see whether pivoting is required (it is
c     the case if withpivoting is nonzero)

      n = mxGetM(prhs(1))
      call mxCopyPtrToInteger4(mxGetPr(prhs(2)),job,1)

c --- Create Fortran array from the input argument.

      allocate(a(n,n))
      n2 = n*n
      call mxCopyPtrToComplex16(mxGetPr(prhs(1)),
     +                          mxGetPi(prhs(1)),a,n2)


c --- Call the computational subroutine after having set its input
c     arguments. Recover its results next

      allocate(work(n))
      allocate(jpvt(n))
      do i=1,n
        jpvt(i) = 0
      enddo

      call zchdc(a,n,n,work,jpvt,job,info)

      if (info .eq. n) then
c       --- info = 0 to say that the factorization has been done
        info = 0
c       --- force the lower triangular part of R to vanish
        do j=1,n
          do i=j+1,n
            a(i,j) = (0.d0,0.d0)
          enddo
        enddo
      endif

      plhs(1) = mxCreateDoubleMatrix(n,n,1)
      call mxCopyComplex16ToPtr(a,mxGetPr(plhs(1)),
     +                            mxGetPi(plhs(1)),n2)

      classid = mxClassIDFromClassName('int32')
      plhs(2) = mxCreateNumericMatrix(n,1,classid,0)
      p_ptr = mxGetPr(plhs(2))
      call mxCopyInteger4ToPtr(jpvt,p_ptr,n)     

      plhs(3) = mxCreateNumericMatrix(1,1,classid,0)
      info_ptr = mxGetPr(plhs(3))
      call mxCopyInteger4ToPtr(info,info_ptr,1)     

c --- Deallocation and return

      deallocate(a)
      deallocate(work)
      deallocate(jpvt)

      return
      end

c=======================================================================
c     Linpack routine zchdc
c=======================================================================

      subroutine zchdc(a,lda,p,work,jpvt,job,info)
      integer lda,p,jpvt(1),job,info
      complex*16 a(lda,1),work(1)
c
c     zchdc computes the cholesky decomposition of a positive definite
c     matrix.  a pivoting option allows the user to estimate the
c     condition of a positive definite matrix or determine the rank
c     of a positive semidefinite matrix.
c
c     on entry
c
c         a      complex*16(lda,p).
c                a contains the matrix whose decomposition is to
c                be computed.  onlt the upper half of a need be stored.
c                the lower part of the array a is not referenced.
c
c         lda    integer.
c                lda is the leading dimension of the array a.
c
c         p      integer.
c                p is the order of the matrix.
c
c         work   complex*16.
c                work is a work array.
c
c         jpvt   integer(p).
c                jpvt contains integers that control the selection
c                of the pivot elements, if pivoting has been requested.
c                each diagonal element a(k,k)
c                is placed in one of three classes according to the
c                value of jpvt(k).
c
c                   if jpvt(k) .gt. 0, then x(k) is an initial
c                                      element.
c
c                   if jpvt(k) .eq. 0, then x(k) is a free element.
c
c                   if jpvt(k) .lt. 0, then x(k) is a final element.
c
c                before the decomposition is computed, initial elements
c                are moved by symmetric row and column interchanges to
c                the beginning of the array a and final
c                elements to the end.  both initial and final elements
c                are frozen in place during the computation and only
c                free elements are moved.  at the k-th stage of the
c                reduction, if a(k,k) is occupied by a free element
c                it is interchanged with the largest free element
c                a(l,l) with l .ge. k.  jpvt is not referenced if
c                job .eq. 0.
c
c        job     integer.
c                job is an integer that initiates column pivoting.
c                if job .eq. 0, no pivoting is done.
c                if job .ne. 0, pivoting is done.
c
c     on return
c
c         a      a contains in its upper half the cholesky factor
c                of the matrix a as it has been permuted by pivoting.
c
c         jpvt   jpvt(j) contains the index of the diagonal element
c                of a that was moved into the j-th position,
c                provided pivoting was requested.
c
c         info   contains the index of the last positive diagonal
c                element of the cholesky factor.
c
c     for positive definite matrices info = p is the normal return.
c     for pivoting with positive semidefinite matrices info will
c     in general be less than p.  however, info may be greater than
c     the rank of a, since rounding error can cause an otherwise zero
c     element to be positive. indefinite systems will always cause
c     info to be less than p.
c
c     linpack. this version dated 03/19/79 .
c     j.j. dongarra and g.w. stewart, argonne national laboratory and
c     university of maryland.
c
c
c     blas zaxpy,zswap
c     fortran dsqrt,dconjg
c
c     internal variables
c
      integer pu,pl,plp1,i,j,jp,jt,k,kb,km1,kp1,l,maxl
      complex*16 temp
      double precision maxdia
      logical swapk,negk
      double precision dreal
      complex*16 zdumr
      dreal(zdumr) = zdumr
c
      pl = 1
      pu = 0
      info = p
      if (job .eq. 0) go to 160
c
c        pivoting has been requested. rearrange the
c        the elements according to jpvt.
c
         do 70 k = 1, p
            swapk = jpvt(k) .gt. 0
            negk = jpvt(k) .lt. 0
            jpvt(k) = k
            if (negk) jpvt(k) = -jpvt(k)
            if (.not.swapk) go to 60
               if (k .eq. pl) go to 50
                  call zswap(pl-1,a(1,k),1,a(1,pl),1)
                  temp = a(k,k)
                  a(k,k) = a(pl,pl)
                  a(pl,pl) = temp
                  a(pl,k) = dconjg(a(pl,k))
                  plp1 = pl + 1
                  if (p .lt. plp1) go to 40
                  do 30 j = plp1, p
                     if (j .ge. k) go to 10
                        temp = dconjg(a(pl,j))
                        a(pl,j) = dconjg(a(j,k))
                        a(j,k) = temp
                     go to 20
   10                continue
                     if (j .eq. k) go to 20
                        temp = a(k,j)
                        a(k,j) = a(pl,j)
                        a(pl,j) = temp
   20                continue
   30             continue
   40             continue
                  jpvt(k) = jpvt(pl)
                  jpvt(pl) = k
   50          continue
               pl = pl + 1
   60       continue
   70    continue
         pu = p
         if (p .lt. pl) go to 150
         do 140 kb = pl, p
            k = p - kb + pl
            if (jpvt(k) .ge. 0) go to 130
               jpvt(k) = -jpvt(k)
               if (pu .eq. k) go to 120
                  call zswap(k-1,a(1,k),1,a(1,pu),1)
                  temp = a(k,k)
                  a(k,k) = a(pu,pu)
                  a(pu,pu) = temp
                  a(k,pu) = dconjg(a(k,pu))
                  kp1 = k + 1
                  if (p .lt. kp1) go to 110
                  do 100 j = kp1, p
                     if (j .ge. pu) go to 80
                        temp = dconjg(a(k,j))
                        a(k,j) = dconjg(a(j,pu))
                        a(j,pu) = temp
                     go to 90
   80                continue
                     if (j .eq. pu) go to 90
                        temp = a(k,j)
                        a(k,j) = a(pu,j)
                        a(pu,j) = temp
   90                continue
  100             continue
  110             continue
                  jt = jpvt(k)
                  jpvt(k) = jpvt(pu)
                  jpvt(pu) = jt
  120          continue
               pu = pu - 1
  130       continue
  140    continue
  150    continue
  160 continue
      do 270 k = 1, p
c
c        reduction loop.
c
         maxdia = dreal(a(k,k))
         kp1 = k + 1
         maxl = k
c
c        determine the pivot element.
c
         if (k .lt. pl .or. k .ge. pu) go to 190
            do 180 l = kp1, pu
               if (dreal(a(l,l)) .le. maxdia) go to 170
                  maxdia = dreal(a(l,l))
                  maxl = l
  170          continue
  180       continue
  190    continue
c
c        quit if the pivot element is not positive.
c
         if (maxdia .gt. 0.0d0) go to 200
            info = k - 1
c     ......exit
            go to 280
  200    continue
         if (k .eq. maxl) go to 210
c
c           start the pivoting and update jpvt.
c
            km1 = k - 1
            call zswap(km1,a(1,k),1,a(1,maxl),1)
            a(maxl,maxl) = a(k,k)
            a(k,k) = dcmplx(maxdia,0.0d0)
            jp = jpvt(maxl)
            jpvt(maxl) = jpvt(k)
            jpvt(k) = jp
            a(k,maxl) = dconjg(a(k,maxl))
  210    continue
c
c        reduction step. pivoting is contained across the rows.
c
         work(k) = dcmplx(dsqrt(dreal(a(k,k))),0.0d0)
         a(k,k) = work(k)
         if (p .lt. kp1) go to 260
         do 250 j = kp1, p
            if (k .eq. maxl) go to 240
               if (j .ge. maxl) go to 220
                  temp = dconjg(a(k,j))
                  a(k,j) = dconjg(a(j,maxl))
                  a(j,maxl) = temp
               go to 230
  220          continue
               if (j .eq. maxl) go to 230
                  temp = a(k,j)
                  a(k,j) = a(maxl,j)
                  a(maxl,j) = temp
  230          continue
  240       continue
            a(k,j) = a(k,j)/work(k)
            work(j) = dconjg(a(k,j))
            temp = -a(k,j)
            call zaxpy(j-k,temp,work(kp1),1,a(kp1,j),1)
  250    continue
  260    continue
  270 continue
  280 continue
      return
      end

c=======================================================================
c     Blas routine zaxpy
c=======================================================================

*> \brief \b ZAXPY
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at
*            http://www.netlib.org/lapack/explore-html/
*
*  Definition:
*  ===========
*
*       SUBROUTINE ZAXPY(N,ZA,ZX,INCX,ZY,INCY)
*
*       .. Scalar Arguments ..
*       COMPLEX*16 ZA
*       INTEGER INCX,INCY,N
*       ..
*       .. Array Arguments ..
*       COMPLEX*16 ZX(*),ZY(*)
*       ..
*
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*>    ZAXPY constant times a vector plus a vector.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee
*> \author Univ. of California Berkeley
*> \author Univ. of Colorado Denver
*> \author NAG Ltd.
*
*> \date December 2016
*
*> \ingroup complex16_blas_level1
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>     jack dongarra, 3/11/78.
*>     modified 12/3/93, array(1) declarations changed to array(*)
*> \endverbatim
*>
*  =====================================================================
      SUBROUTINE ZAXPY(N,ZA,ZX,INCX,ZY,INCY)
*
*  -- Reference BLAS level1 routine (version 3.7.0) --
*  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     December 2016
*
*     .. Scalar Arguments ..
      COMPLEX*16 ZA
      INTEGER INCX,INCY,N
*     ..
*     .. Array Arguments ..
      COMPLEX*16 ZX(*),ZY(*)
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER I,IX,IY
*     ..
*     .. External Functions ..
      DOUBLE PRECISION DCABS1
      EXTERNAL DCABS1
*     ..
      IF (N.LE.0) RETURN
      IF (DCABS1(ZA).EQ.0.0d0) RETURN
      IF (INCX.EQ.1 .AND. INCY.EQ.1) THEN
*
*        code for both increments equal to 1
*
         DO I = 1,N
            ZY(I) = ZY(I) + ZA*ZX(I)
         END DO
      ELSE
*
*        code for unequal increments or equal increments
*          not equal to 1
*
         IX = 1
         IY = 1
         IF (INCX.LT.0) IX = (-N+1)*INCX + 1
         IF (INCY.LT.0) IY = (-N+1)*INCY + 1
         DO I = 1,N
            ZY(IY) = ZY(IY) + ZA*ZX(IX)
            IX = IX + INCX
            IY = IY + INCY
         END DO
      END IF
*
      RETURN
      END

c=======================================================================
c     Blas routine zswap
c=======================================================================

*> \brief \b ZSWAP
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at
*            http://www.netlib.org/lapack/explore-html/
*
*  Definition:
*  ===========
*
*       SUBROUTINE ZSWAP(N,ZX,INCX,ZY,INCY)
*
*       .. Scalar Arguments ..
*       INTEGER INCX,INCY,N
*       ..
*       .. Array Arguments ..
*       COMPLEX*16 ZX(*),ZY(*)
*       ..
*
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*>    ZSWAP interchanges two vectors.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee
*> \author Univ. of California Berkeley
*> \author Univ. of Colorado Denver
*> \author NAG Ltd.
*
*> \date December 2016
*
*> \ingroup complex16_blas_level1
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>     jack dongarra, 3/11/78.
*>     modified 12/3/93, array(1) declarations changed to array(*)
*> \endverbatim
*>
*  =====================================================================
      SUBROUTINE ZSWAP(N,ZX,INCX,ZY,INCY)
*
*  -- Reference BLAS level1 routine (version 3.7.0) --
*  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     December 2016
*
*     .. Scalar Arguments ..
      INTEGER INCX,INCY,N
*     ..
*     .. Array Arguments ..
      COMPLEX*16 ZX(*),ZY(*)
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
      COMPLEX*16 ZTEMP
      INTEGER I,IX,IY
*     ..
      IF (N.LE.0) RETURN
      IF (INCX.EQ.1 .AND. INCY.EQ.1) THEN
*
*       code for both increments equal to 1
         DO I = 1,N
            ZTEMP = ZX(I)
            ZX(I) = ZY(I)
            ZY(I) = ZTEMP
         END DO
      ELSE
*
*       code for unequal increments or equal increments not equal
*         to 1
*
         IX = 1
         IY = 1
         IF (INCX.LT.0) IX = (-N+1)*INCX + 1
         IF (INCY.LT.0) IY = (-N+1)*INCY + 1
         DO I = 1,N
            ZTEMP = ZX(IX)
            ZX(IX) = ZY(IY)
            ZY(IY) = ZTEMP
            IX = IX + INCX
            IY = IY + INCY
         END DO
      END IF
      RETURN
      END

c=======================================================================
c     Blas routine dcabs1
c=======================================================================

*> \brief \b DCABS1
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at
*            http://www.netlib.org/lapack/explore-html/
*
*  Definition:
*  ===========
*
*       DOUBLE PRECISION FUNCTION DCABS1(Z)
*
*       .. Scalar Arguments ..
*       COMPLEX*16 Z
*       ..
*       ..
*
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DCABS1 computes |Re(.)| + |Im(.)| of a double complex number
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee
*> \author Univ. of California Berkeley
*> \author Univ. of Colorado Denver
*> \author NAG Ltd.
*
*> \date December 2016
*
*> \ingroup double_blas_level1
*
*  =====================================================================
      DOUBLE PRECISION FUNCTION DCABS1(Z)
*
*  -- Reference BLAS level1 routine (version 3.7.0) --
*  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     December 2016
*
*     .. Scalar Arguments ..
      COMPLEX*16 Z
*     ..
*     ..
*  =====================================================================
*
*     .. Intrinsic Functions ..
      INTRINSIC ABS,DBLE,DIMAG
*
      DCABS1 = ABS(DBLE(Z)) + ABS(DIMAG(Z))
      RETURN
      END
