function [A,b,c,x0,y0,cname] = matpower_qcqpc(testcase,fout)

%%
% [A,b,c,x0,y0,cname] = matpower_qcqpc(testcase,fout)
%
% Read the datafile 'matpower_TESTCASE_qcqp.txt' (where 'TESTCASE'
% stands for the Matpower 'testcase' specified on entry) and return the
% data in A, b, c, x0, y0, in complex numbers in a form expected by
% sdolab 0.4. The cell of strings 'cname.r' and 'cname.c' give the names
% of the constraints in the order they appear in A.r (cname.r) and A.c
% (cname.c).

  A  = [];
  b  = [];
  c  = [];
  x0 = [];
  y0 = [];

% Constants

  eps_hermit = 1.e-12;	% threshold determining whether a matrix is Hermitian ('ishermitian' does not exist)

% Print

  fprintf(fout,'\nMatpower ''%s'' (complex numbers) (full data)\n',testcase);

%-----------------------------------------------------------------------
% Read datafile
%-----------------------------------------------------------------------

% Check existence of the datafile

  datafile = strcat('matpower_',lower(testcase),'_qcqp_c.txt');
  fprintf(fout,'\nGetting data from the datafile ''%s''',datafile);
  if exist(datafile) ~= 2
    fprintf(fout,'\n### datafile ''%s'' not found\n\n',datafile);
    return
  end

  fid = fopen(datafile,'r');

% Dimensions

  n0 = sscanf(fgets(fid),'%i',1);	% size of the matrix to find in the QCQP problem
  m0 = sscanf(fgets(fid),'%i',1);	% number of equality constraints
  p0 = sscanf(fgets(fid),'%i',1);	% number of inequality constraints
  fprintf(fout,'\n. number of variables %17i',n0);

% Objective

  nnz = sscanf(fgets(fid),'%i',1);
  CC = sparse(n0,n0);			% matrix of the objective
  for l = 1:nnz
    i = fscanf(fid,'%i',1);
    j = fscanf(fid,'%i',1);
    CC(i,j) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
  end
  fgets(fid);				% next line to finish the line for the next fgets

  cc = sscanf(fgets(fid),'%e',1);	% constant term in the objective

  % check objective

  Cn = norm(CC,'fro');
  if Cn == 0; Cn = 1; end
  Hdiff = norm(CC-CC','fro')/Cn;
  if Hdiff > eps_hermit
    fprintf(fout,'\n>>> the matrix objective matrix is not Hermitian: |C-C''|/|C| = %12.2e\n\n',Hdiff);
    return
  end

% Equality constraints

  AA = cell(m0,1);			% cell of matrices of the equality constraints
  for k=1:m0
    nnz = sscanf(fgets(fid),'%i',1);
    AAA = sparse(n0,n0);
    for l = 1:nnz
      i = fscanf(fid,'%i',1);
      j = fscanf(fid,'%i',1);
      AAA(i,j) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
    end
    fgets(fid);				% next line to finish the line for the next fgets
    AA{k} = AAA;
  end

  aa = zeros(m0,1);			% RHS for the equality constraints
  aa(1) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
  fgets(fid);				% next line to finish the line for the next fgets
  for k=2:m0
    aa(k) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
  end
  if m0>1; fgets(fid); end		% next line to finish the line for the next fgets

% Inequality constraints

  BB = cell(p0,1);			% matrices of the inequality constraints
  for k=1:p0
    nnz = sscanf(fgets(fid),'%i',1);
    BBB = sparse(n0,n0);
    for l = 1:nnz
      i = fscanf(fid,'%i',1);
      j = fscanf(fid,'%i',1);
      BBB(i,j) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
    end
    fgets(fid);				% next line to finish the line for the next fgets
    BB{k} = BBB;
  end

  bb = zeros(p0,1);			% RHS for the inequality constraints
  bb(1) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
  fgets(fid);				% next line to finish the line for the next fgets
  for k=2:p0
    bb(k) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
  end
  if p0>1; fgets(fid); end		% next line to finish the line for the next fgets

% Initial x0 and y0

  x_is_known = false;
  y_is_known = false;

  tline = fgets(fid);

  if tline ~= -1			% read initial X

    x_is_known = true;
    nnz = sscanf(tline,'%i',1);
    xx = sparse(n0,n0);
    for l = 1:nnz
      i = fscanf(fid,'%i',1);
      j = fscanf(fid,'%i',1);
      xx(i,j) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
    end

    fgets(fid);				% next line to finish the line for the next fgets
    tline = fgets(fid);

    if tline ~= -1			% read initial y in complex numbers

      y_is_known = true;
      nnz = sscanf(tline,'%i',1);
      yy = sparse(m0+p0,1);
      for l = 1:nnz
        i = fscanf(fid,'%i',1);
        yy(i) = complex(fscanf(fid,'%e',1),fscanf(fid,'%e',1));
      end

    end

  end

  fclose(fid);

%-----------------------------------------------------------------------
% Transform the complex data in the one accepted by SDOLAB 0.4 (in
% complex numbers). Inequality constraints are transformed into equality
% constraints using slag variables. These slag variables are joined to
% the unknown matrix XX to form a new larger unknown matrix X containing
% XX and the slag variables on its diagonal.
%-----------------------------------------------------------------------

% Check consistency of the equality constraints (if AA{k} is Hermitian, imag(aa(k)) cannot be nonzero) and update mr and mc

  mr = 0;
  mc = 0;
  for k=1:m0
    An = norm(AA{k},'fro');
    if An == 0; An = 1; end
    Hdiff = norm(AA{k}-AA{k}','fro')/An;
    if Hdiff > eps_hermit
      aiscomplex(k) = true;
      mc = mc+1;
    else
      aiscomplex(k) = false;
      mr = mr+1;
      if ~isreal(aa(k))
        fprintf(fout,'\n>>> A(%i) is Hermitian but a(%i) is a complex number\n\n',k,k);
        return
      end
    end
  end

  fprintf(fout,'\n. number of equality constraints %6i (%i real, %i complex)',m0,mr,mc);

% Check consistency of the inequality constraints (if BB{k} is Hermitian, imag(bb(k)) cannot be negative), update mr and mc, and
% determine the number ns of real slacks for the inequalities

  mri = 0;
  mci = 0;
  for k=1:p0
    Bn = norm(BB{k},'fro');
    if Bn == 0; Bn = 1; end
    Hdiff = norm(BB{k}-BB{k}','fro')/Bn;
    if Hdiff > eps_hermit
      biscomplex(k) = true;	% never mind if bb(k) is real
      mci = mci+1;
    else
      biscomplex(k) = false;
      mri = mri+1;
      if ~isreal(bb(k)) & (imag(bb(k)) < 0)
        fprintf(fout,'\n>>> B(%i) is Hermitian but b(%i) has a negative imaginary part (%12.5e)\n\n',k,k,imag(bb(k)));
        return
      end
    end
  end

  mr = mr+mri;		% nb of real (or Hermitian) equality of inequality constraints
  mc = mc+mci;		% nb of complex (or non Hermitian) equality of inequality constraints
  ns = mri+2*mci;	% nb of slack variables

  cname.r = cell(mr,1);
  cname.c = cell(mc,1);

  fprintf(fout,'\n. number of inequality constraints %4i (%i real, %i complex)\n',p0,mri,mci);

  n2  = (n0+ns)^2; 		% the standard SDO matrix is of size n0+ns
  mc  = mc+(n2-n0^2-ns)/2;	% number of affine complex constraints: mc (computed before) + number of zeroing constraints (n2-n0^2-ns)
  A.r = zeros(mr,n2);
  A.c = zeros(mc,n2);
  b.r = zeros(mr,1);
  b.c = zeros(mc,1);

% Equality constraints. The data file gives a matrix Ak (complex or Hermitian), which is presently in AA{k}, such that the
% associated k-th constraint reads x'*Ak*x = a, where x' is the conjugate transpose of x. In the relaxation of the problem,
% this constraint becomes <Ak',X> = a, where Ak' is the conjugate transpose of Ak and X is a Hermitian matrix. Now <Ak',X> =
% tr(Ak*X) = sum_{ij} Ak_{ij}*X_{ji} = sum_{ij} (Ak.')_{ij}*X_{ij}, where Ak.' denotes the transpose of Ak. Since this quantity
% is evaluated in the code by A(k,:)*x, where x = vec(X), the k-th row A(k,:) of A must contain AA{k}.'.

  kr = 0;
  kc = 0;
  for k = 1:m0
    if aiscomplex(k)
      kc = kc+1;
      A.c(kc,:) = vec([AA{k}.', zeros(n0,ns); zeros(ns,n0+ns)]);
      b.c(kc)   = aa(k);
      cname.c{kc} = sprintf('A%i',k);
    else
      kr = kr+1;
      A.r(kr,:) = vec([AA{k}.', zeros(n0,ns); zeros(ns,n0+ns)]);
      b.r(kr)   = aa(k);
      cname.r{kr} = sprintf('A%i',k);
    end
  end

% Inequality constraints

  zzz = sparse(ns,ns);
  kz  = 0;	% pointer on the diagonal of zzz
  for k = 1:p0
    if biscomplex(k)
      kc = kc+1;
      kz = kz+1;
      zzz(kz,kz) = 1;
      kz = kz+1;
      zzz(kz,kz) = 1i;
      A.c(kc,:) = vec([BB{k}.', zeros(n0,ns); zeros(ns,n0), zzz]);
      b.c(kc)   = bb(k);
      cname.c{kc} = sprintf('B%i',k);
      kz = kz-1;
      zzz(kz,kz) = 0;
      kz = kz+1;
      zzz(kz,kz) = 0;
    else
      kr = kr+1;
      kz = kz+1;
      zzz(kz,kz) = 1;
      A.r(kr,:) = vec([BB{k}.', zeros(n0,ns); zeros(ns,n0), zzz]);
      b.r(kr)   = bb(k);
      cname.r{kr} = sprintf('B%i',k);
      zzz(kz,kz) = 0;
    end
  end

% Add equality constraints to impose zeros in the Hermitian matrix X (above and in the LHS of the added diagonal). This cannot
% be done by <A,X>=0 with a Hermitian matrix A, whose nonzero elements are A_{ij}=:a and A_{ji}=conj(a), since then, denoting
% x:=X_{ij}, the constraint reads conj(a)*x+a*conj(x)=0 or real(conj(a)*x)=0, which cannot imply that x=0 (a single real
% constraint, while two are necessary). For this reason, we impose <A,X>=0 with a non symmetric (and non Hermitian) real matrix
% A, whose single nonzero element is A_{ij}=1. The RHS was set to zero by initialization.

  zzz = sparse(n0+ns,n0+ns);	% constraint matrix (zero at the begining)

  for i = n0+1:n0+ns
    for j = 1:i-1
      kc = kc+1;
      zzz(i,j) = 1;
      A.c(kc,:) = vec(zzz);
      cname.c{kc} = sprintf('X(%i,%i)=0',i,j);
      zzz(i,j) = 0;
    end
  end

% Objective

  c = full(vec([CC  sparse(n0,ns); sparse(ns,n0+ns)]));

% Initial x0

  if x_is_known
    kx = n0;	% pointer on the diagonal of x0
    x0 = zeros(n0+mri+2*mci,n0+mri+2*mci);
    x0(1:n0,1:n0) = xx;
    for k = 1:p0
      if biscomplex(k)
        tmp = bb(k)-vec(BB{k}.').'*vec(xx);
        kx = kx+1;
        x0(kx,kx) = real(tmp);
        kx = kx+1;
        x0(kx,kx) = imag(tmp);
      else
        kx = kx+1;
        x0(kx,kx) = bb(k)-vec(BB{k}.').'*vec(xx);
      end
    end
    x0 = vec(x0);
  end

% Initial y0

  if y_is_known
    kr = 0;
    kc = 0;
    y0.r = zeros(mr,1);
    y0.c = zeros(mc,1);
    for k=1:m0
      if aiscomplex(k)
        kc = kc+1;
        y0.c(kc) = yy(k);
      else
        kr = kr+1;
        y0.r(kc) = yy(k);
      end
    end
    for k=1:p0
      if biscomplex(k)
        kc = kc+1;
        y0.c(kc) = yy(m0+k);
      else
        kr = kr+1;
        y0.r(kr) = yy(m0+k);
      end
    end
  end

  return
