function [A,b,c,x0,y0] = mmnc(p1,p2,q,r,fout)

%%
% [A,b,c,x0,y0] = mmnc(p1,p2,q,r,fout)
%
% Minimum matrix norm problem in complex numbers, which requires to find
% p1 real numbers z(1), ..., z(p1) and p2 complex numbers z(p1+1), ...,
% z(p1+p2) such that B0 + z(1)*B1 + z(p)*Bp, with p=p1+p2, is of minimum
% euclidean norm. The complex matrices B0, ..., Bp have size (q,r).
%
% On return, A and b are structures such that A.r'*x = b.r are the real
% affine constraints of the SDO problem and A.c'*x = b.c are its complex
% affine constraints. Furthermore c'*x is the cost function of the
% problem. Finally mat(x0) is a strictly feasible primal matrix and
% mat(c-A.r'*y0.r-A.c'*y0.c) is a stricly feasible dual matrix.
%
% References:
% - L. Vandenberghe and S. Boyd (1996), "Semidefinite programming", SIAM
%   Review, 38, 49–95, for the presentation of problem in real numbers
%   (they may be other references) and its SDO formulation,
% - J.Ch. Gilbert and C. Josz (2016), "Plea for a semidefinite
%   optimization solver in complex numbers", for an adaptation to the
%   complex setting.

% Print

  fprintf(fout,'\nMinimum matrix norm problem in complex numbers');
  fprintf(fout,'\n. %i matrices of size (%i,%i)\n. %i real weight(s)\n. %i complex weight(s)\n',p1+p2+1,q,r,p1,p2);

% Check entry

  if (p1 < 0) | (p2 < 0)
    fprintf(fout,'\n>>> mmnc: p1 and p2 must be nonnegative\n\n');
    return
  end
  if (q <= 0) | (r <= 0)
    fprintf(fout,'\n>>> mmnc: q and r must be positive\n\n');
    return
  end
  if p1+2*p2 > 2*q*r
    fprintf(fout,'\n### mmnc: too many variables compared to matrix size for getting constraint surjectivity');
    fprintf(fout,'\n### mmnc: (%i = p1+2*p2 > 2*q*r = %i, with p1=%i, p2=%i, q=%i, r=%i)\n',p1+2*p2,2*q*r,p1,p2,q,r);
  end

% Get the data

  % Set the random generator seed

  rng(0);

  % Generate the random complex matrices B

  p = p1+p2;
  B = zeros(q,r,p+1);
  for i = 1:p+1
    B(:,:,i) = rand(q,r)+1i*rand(q,r);
  end

% Dimensions

  n  = q+r;
  n2 = n^2;
  m  = p+1;

% Put the data in standard form

  % real constraints

  A.r  = zeros(p1+1,n2);
  b.r  = zeros(p1+1,1);
  A.r(1,:) = vec(-eye(n))';
  for i = 2:p1+1
    A.r(i,:) = -vec([zeros(q), conj(B(:,:,i)); B(:,:,i).', zeros(r)])';
  end
  b.r(1)  = -1;

  % complex constraints

  if p2
    A.c = zeros(p2,n2);
    for i = 1:p2
      A.c(i,:) = -vec([zeros(q), 2*conj(B(:,:,p1+1+i)); zeros(r,n)])';
    end
    b.c  = zeros(p2,1);
  else
    A.c = [];
    b.c = [];
  end

  % objective

  c = vec([zeros(q), conj(B(:,:,1)); B(:,:,1).', zeros(r)]);

% Initial strictly feasible point

  % strictly feasible point

% x0    = vec(eye(n))/n;
% y0    = zeros(m,1);
% y0(1) = norm(B(:,:,1))+1;

  % strictly feasible point in the 1/3-neighborhood

  x0    = vec(eye(n))/n;
  y0.r = zeros(p1+1,1);
  y0.r(1) = norm(B(:,:,1))*n*1.5;
  if p2
    y0.c = zeros(p2,1);
  else
    y0.c = [];
  end

return
